import os
import unittest

from pysecretary import pysecretary, InvalidPrefixError, UnsupportedPrefixError
from pysecretary.exceptions import NotFoundError


class TestPysecretary(unittest.TestCase):
    def test_env_get(self):
        os.environ["PYSECRETARY"] = "test"
        t = pysecretary.get("env://PYSECRETARY")
        self.assertEqual(t, "test")

    def test_env_raise(self):
        os.environ.clear()
        with self.assertRaises(NotFoundError):
            pysecretary.get("env://PYSECRETARY")

    def test_unsupported_prefix(self):
        with self.assertRaises(UnsupportedPrefixError):
            pysecretary.get("invalid://PYSECRETARY")

    def test_invalid_prefix(self):
        with self.assertRaises(InvalidPrefixError):
            pysecretary.register("", lambda x, y: x)

        with self.assertRaises(InvalidPrefixError):
            pysecretary.get("PYSECRETARY")

        with self.assertRaises(InvalidPrefixError):
            pysecretary.get("v:/PYSECRETARY")

    def test_with_default(self):
        t = pysecretary.get("env://PYSECRETARY", 'default')
        self.assertEqual(t, 'default')

        t = pysecretary.get("PYSECRETARY", 'default')
        self.assertEqual(t, 'default')



"""Top-level package for PySecretary."""
from .exceptions import PysecretaryError, UnsupportedPrefixError, InvalidPrefixError
from .pysecretary import (
    get,
    register,
)

__author__ = """Spencer McCreary"""
__email__ = "spencer_mccreary@compulsivesoftware.com"
__version__ = "0.1.4"

__all__ = [
    "get",
    "register",
    "UnsupportedPrefixError",
    "InvalidPrefixError",
    "PysecretaryError",
]



import os

from .exceptions import NotFoundError


class SecretRetriever:

    def get(self, key: str) -> str:
        """
           Retrieves the contents of *key*
           :param str key: key passed into pysecretary with the prefix stripped
        """
        raise NotImplementedError


class EnvironmentSecretRetriever(SecretRetriever):

    def get(self, key: str) -> str:
        value = os.environ.get(key)
        if value is None:
            raise NotFoundError(f'no environment variable found for key "{key}"')
        return value



import re
from typing import Tuple

from pysecretary import InvalidPrefixError

PREFIX_PATTERN = re.compile(r"(^(\w+):\/\/)")
VALID_PREFIX = re.compile(r"^\w+")


def parse(v: str) -> Tuple[str, str]:
    elements = re.split(PREFIX_PATTERN, v, maxsplit=1)
    if len(elements) == 4:
        prefix = elements[2].casefold()
        value = elements[3]

        return prefix, value

    raise InvalidPrefixError(v)



from typing import Dict, Set

from .exceptions import InvalidPrefixError, UnsupportedPrefixError
from .prefix import VALID_PREFIX
from .retriever import SecretRetriever


class SecretRetrieverRegistry:

    def __init__(self):
        self._supported_prefixes: Set[str] = set()
        self._registry: Dict[str, SecretRetriever] = dict()

    def register(self, prefix: str, retriever: SecretRetriever, force=False):
        if not VALID_PREFIX.match(prefix):
            raise InvalidPrefixError(
                f'prefix {prefix} is not valid, a prefix must be in the format of {VALID_PREFIX}.')

        if not isinstance(retriever, SecretRetriever):
            raise TypeError(f' "{type(retriever)}" is not a SecretRetriever')

        if prefix in self._supported_prefixes and not force:
            raise KeyError(
                f'prefix "{prefix}" already exists, use force=True if you want to overwrite'
            )

        self._supported_prefixes.add(prefix)
        self._registry[prefix] = retriever

    def is_supported(self, prefix: str) -> bool:
        if prefix not in self._supported_prefixes:
            return False
        return True

    def get(self, prefix: str) -> SecretRetriever:
        if not self.is_supported(prefix):
            raise UnsupportedPrefixError()
        return self._registry.get(prefix)



from google.cloud.secretmanager_v1 import SecretManagerServiceClient

from pysecretary.retriever import SecretRetriever


class GCPSecretRetriever(SecretRetriever):
    def __init__(self, client: SecretManagerServiceClient):
        self._client = client

    def get(self, key: str):
        request = {
            "name": key
        }
        response = self._client.access_secret_version(request=request)
        return response.payload.data.decode("UTF-8")



class PysecretaryError(Exception):
    pass


class UnsupportedPrefixError(PysecretaryError):
    """
    A value contains a prefix that is not supported
    """


class NotFoundError(PysecretaryError):
    """
    Could not find value
    """


class InvalidPrefixError(PysecretaryError):
    """
    A value does not have a valid prefix
    """



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

with open("README.rst") as readme_file:
    readme = readme_file.read()

with open("HISTORY.rst") as history_file:
    history = history_file.read()

requirements = []

setup(
    author="Spencer McCreary",
    author_email="spencer_mccreary@compulsivesoftware.com",
    python_requires=">=3.5",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    description="PySecretary is a python secrets library",
    install_requires=requirements,
    license="MIT license",
    long_description=readme + "\n\n" + history,
    include_package_data=True,
    keywords="pysecretary",
    name="pysecretary",
    packages=find_packages(include=["pysecretary", "pysecretary.*"]),
    url="https://github.com/spennymac/pysecretary",
    version="0.1.5",
    zip_safe=False,
)



from .exceptions import PysecretaryError
from .prefix import parse
from .registry import SecretRetrieverRegistry
from .retriever import SecretRetriever, EnvironmentSecretRetriever


class NotFound:
    pass


NOT_FOUND = NotFound()


def get(v: str, default=NOT_FOUND) -> str:
    try:
        prefix, value = parse(v)
        retriever: SecretRetriever = _registry.get(prefix)
        return retriever.get(value)
    except PysecretaryError:
        if default is NOT_FOUND:
            raise
        return default


def register(prefix: str, retriever: SecretRetriever, force=False):
    _registry.register(prefix, retriever, force)


_registry = SecretRetrieverRegistry()
_registry.register("env", EnvironmentSecretRetriever())
